/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <dirent.h>
#include <stdarg.h>


#include "main.h"
#include "config.h"
#include "misc.h"
#include "fs.h"
#include "strings.h"
#include "xml.h"

int ovcp_write(struct ovcp_session *session, char *data, int len)
{
        int ret=-1, transfered=0;

        
	#ifdef HAVE_LIBGNUTLS        
	if(session->tls_session != NULL)
	{
	
	 	while(transfered < len)
		{
			ret = gnutls_record_send (*(session->tls_session), data+transfered, len-transfered);

			if(ret < 0) break;
			else transfered += ret;
		}
	}	
        else
	#endif
		ret = write(session->session_socket, data, len);
		
	return ret;
}

int ovcp_read(struct ovcp_session *session, char *data, int len)
{
        int ret;
        
	#ifdef HAVE_LIBGNUTLS
	if(session->tls_session != NULL)
	{	
		ret = gnutls_record_recv (*(session->tls_session), data, len);
	}	
	else
	#endif
                ret = read(session->session_socket, data, len);

	return ret;
}

char *ovcp_readline(struct ovcp_session *session)
{
	int i = 0;
	int ret;
	char c = 0;
	char *buffer = NULL;

	while(i < 15000)
	{
                #ifdef HAVE_LIBGNUTLS
		if(session->tls_session != NULL)
                      ret = gnutls_record_recv(*(session->tls_session), &c, 1);     
                else
                #endif
		      ret = read(session->session_socket, &c, 1);

		if(ret != 1 || c == '\n' || c == '\r')
			break;

		buffer = realloc(buffer, i+2);
		buffer[i] = c;

		i++;
	}

	if(buffer != NULL)
		buffer[i] = 0;

	return buffer;
}

int ovcp_close(struct ovcp_session *session)
{

        #ifdef HAVE_LIBGNUTLS

        if(session->tls_session != NULL)
        {
                gnutls_bye (*(session->tls_session), GNUTLS_SHUT_WR);
                gnutls_deinit (*(session->tls_session));
                free(session->tls_session);
        }

        #endif

        close(session->session_socket);
        
        free(session);
        
        return 0;
}




#ifdef HAVE_LIBGNUTLS

gnutls_session_t *tls_session_new (int client_socket, gnutls_anon_server_credentials_t anoncred)
{
        gnutls_session_t *session;
        const int kx_prio[] = { GNUTLS_KX_ANON_DH, 0 };

        session = (gnutls_session_t *)malloc(sizeof(gnutls_session_t));

        gnutls_init (session, GNUTLS_SERVER);

        gnutls_set_default_priority (*session);
        gnutls_kx_set_priority (*session, kx_prio);

        gnutls_credentials_set (*session, GNUTLS_CRD_ANON, anoncred);

        gnutls_dh_set_prime_bits (*session, 1024);

        gnutls_transport_set_ptr (*session, (gnutls_transport_ptr_t) client_socket);

        return session;
}

void gnutls_log(int level, const char *msg)
{

	 printf("[GNUTLS] [%d] [%s]\n", level, msg);

}

#endif






int vserver_xid(char *name)
{
        char *path, *context_str;
        int xid = 0;

        path = concat(global_settings.config_dir, "/", name, "/context", NULL);

        context_str = file_read(path);

        if(context_str != NULL)
        {
                trim_string(context_str);
                xid = atoi(context_str);

                free(context_str);
        }

        free(path);

        return xid;
}

char *vserver_name(int xid)
{
	DIR *vconfigdir;
	char path[BUF_SIZE];
	struct dirent *vsentry;
	char *xidstr, *name = NULL;
	
	vconfigdir = opendir(global_settings.config_dir);

        while((vsentry = readdir(vconfigdir)) != NULL)
	{
		if(vsentry->d_name[0] == '.')
			continue;
			
		sconcat(path, BUF_SIZE, global_settings.config_dir, "/", vsentry->d_name, "/context", NULL);
		xidstr = file_read(path);
		
		if(xidstr == NULL)
			continue;
		
		if(xid == atoi(xidstr))
		{
			free(xidstr);
			name = strdup(vsentry->d_name);
			break;
		}
		
		free(xidstr);					
        }
        
        closedir(vconfigdir);

        return name;
}

int vserver_exist(char *name)
{

        char *path;

        path = concat(global_settings.config_dir, "/", name, NULL);

        DIR *dirst = opendir(path);

        free(path);

        if(dirst != NULL)
        {
                closedir(dirst);
                return 1;
        }


        return 0;

}
int vserver_lock(char *name)
{
        char *path;

        path = concat(global_settings.config_dir, "/", name, "/locked", NULL);

        if(!file_exist(path))
        {
                file_touch(path);
                free(path);
        }
        else
        {
                free(path);
                return -1;
        }

        return 1;
}


int vserver_unlock(char *name)
{
        char *path;

        path = concat(global_settings.config_dir, "/", name, "/locked", NULL);

        unlink(path);
        free(path);

        return 1;
}


char *exec_cmd(char *cmd, int exectime, ...)
{
        char *output;

        char **argv = NULL, *next_arg;
        va_list ap;
        int i = 0;

        va_start(ap, exectime);


        while((next_arg = va_arg(ap, char *)) != NULL)
        {
                argv = (char **)realloc(argv, sizeof(char **) * (i+2));

                argv[i] = next_arg;

                i++;
        }

        argv[i] = NULL;

        va_end(ap);

        output = exec_cmdv(cmd, exectime, argv);

        free(argv);

        return output;
}


char *exec_cmdv(char *cmd, int exectime, char **argv)
{

        int pipe_fds[2];
        char *output = NULL;
        int res, len = 0, ges = 0;

        struct timeval timeout;
        fd_set fdset;



        pipe(pipe_fds);

        if(fork() == 0)
        {
                close(1);
                close(2);
                dup(pipe_fds[1]);
                dup(pipe_fds[1]);
                close(pipe_fds[0]);
                close(pipe_fds[1]);

                execvp(cmd, argv);

                perror("execvp");
                exit(-1);
        }
        close(pipe_fds[1]);

        do
        {
                len = 0;

                FD_ZERO(&fdset);
                FD_SET(pipe_fds[0], &fdset);

                timeout.tv_sec = exectime;
                timeout.tv_usec = 0;


                res = select(pipe_fds[0]+1, &fdset, NULL, NULL, &timeout);

                if(res != 0 && res != -1)
                {
                        output = (char *)realloc(output, ges+1000+1);
                        len = read(pipe_fds[0], output+ges, 1000);

                        ges += len;
                        output[ges] = 0;
                }
        }
        while(len != 0);

        close(pipe_fds[0]);

	if(output != NULL)
	{
		if(strlen(output) == 0)
		{
			free(output);
			output = NULL;
		}
		else
		{
        		output = (char *)realloc(output, ges+1);
		}
	}



        return output;
}




int die(char *string)
{
        printf("%s", string);
        exit(-1);
}


char **parse_ips(char *name, int *num)
{
        int counter = 0;
        struct dirent *dir_info;
        char *path, *ip;
        DIR *dirst;
	char *subnet;
		
        char **output = NULL;
        *num = 0;


        path = concat(global_settings.config_dir, "/", name,"/interfaces", NULL);
        dirst = opendir(path);
        free(path);

        if(dirst != NULL)
        {

                while ((dir_info = readdir(dirst)) != NULL)
                {

                        if(strcmp(dir_info->d_name, ".") != 0 && strcmp(dir_info->d_name, "..") != 0)
                        {

                                path = concat(global_settings.config_dir, "/", name,"/interfaces/", dir_info->d_name, NULL);

                                if(dir_exist(path))
                                {
                                        free(path);
                                        path = concat(global_settings.config_dir, "/", name,"/interfaces/", dir_info->d_name, "/ip", NULL);

                                        ip = file_read(path);
                                        if(ip != NULL)
                                        {
                                                trim_string(ip);
						subnet = strstr(ip, "/");
						if(subnet != NULL)
							subnet[0] = 0;
							
                                                output = realloc(output, (counter+1) * sizeof(char **));
                                                output[counter] = ip;

                                                #ifdef DEBUG
                                                        printf("IP %s:%d: %s\n", name, counter, output[counter]);
                                                #endif

                                                counter++;
                                        }
                                }

                                free(path);

                        }

                }

                *num = counter;
                closedir(dirst);
        }

        return output;
}


struct vsip_entry *vserver_iplist(void)
{
	struct vsip_entry *allips = NULL;
	char **vsips;
	int i, entrysnum, allentrys = 0;
	
	DIR *dir_ptr;
	struct dirent *dir_entry;
	char path[BUF_SIZE];



	if((dir_ptr = opendir(global_settings.config_dir)) != NULL)
	{
		
		while((dir_entry = readdir(dir_ptr)) != NULL)
		{
			

			if(dir_entry->d_name[0] != '.')
			{
				sconcat(path, BUF_SIZE, global_settings.config_dir, "/", dir_entry->d_name, NULL);

				if(dir_exist(path))
				{
					i = 0;
					vsips = parse_ips(dir_entry->d_name, &entrysnum);
					
					while(i<entrysnum)
					{
						
						allips = realloc(allips, sizeof(struct vsip_entry)*(allentrys+2));
						allips[allentrys].name = strdup(dir_entry->d_name);
						allips[allentrys].ip = strdup(vsips[i]);
						
						allentrys++;
						i++;
					}	
					
					free_strings(vsips, entrysnum);	

				}
			}
	
		}


		closedir(dir_ptr);
	}
	
	if(allips != NULL)
	{
		allips[allentrys].name = NULL;
		allips[allentrys].ip = NULL;
	}
	
	return allips;
}

int free_iplist(struct vsip_entry *list)
{
	int i=0;
	
	if(list == NULL)
		return -1;
	
	while(list[i].name != NULL && list[i].ip != NULL)
	{
		free(list[i].name);
		free(list[i].ip);
	
		i++;
	}

	free(list);

	return 0;
}

int vci_version(void)
{
	FILE *fd;
	char *tag, *value;
	int part1, part2;
	int version = -1;
	
	fd = fopen("/proc/virtual/info", "r");
	
	if(fd != NULL)
	{
		while(!feof(fd) && version == -1)
		{
			fscanf(fd, "%as %as\n", &tag, &value);
			
			if(strcmp(tag, "VCIVersion:") == 0)
			{

				sscanf(value, "%x:%x", &part1, &part2);
				version = part1<<16;
				version = version | part2;
				
			}


			
			free(tag);
			free(value);
		}
		
		fclose(fd);
	}		

			

	
	return version;
}
