/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/statvfs.h>

#include "node.h"
#include "strings.h"
#include "fs.h"
#include "config.h"
#include "xml.h"
#include "error.h"
#include "parseconfig.h"


struct ovcp_response_st *node_version(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	
	response = ovcp_response_new();

	ovcp_response_add_string(response, PACKAGE_VERSION);
	
	return response;	
}

struct ovcp_response_st *node_halt(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	int ret;
	
	response = ovcp_response_new();
		
	if((ret = fork()) == 0)
	{
		execlp ("shutdown", "shutdown", "-h", "+1", NULL);
		exit(0);
	}
	else
	if(ret == -1)
	{
		ovcp_response_add_boolean(response, FALSE);
	}
	else
	{
		ovcp_response_add_boolean(response, TRUE);
	}
	
	return response;	
}

struct ovcp_response_st *node_reboot(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	int ret;
	
	response = ovcp_response_new();
		
	if((ret = fork()) == 0)
	{
		execlp ("shutdown", "shutdown", "-r", "+1", NULL);
		exit(0);
	}
	else
	if(ret == -1)
	{
		ovcp_response_add_boolean(response, FALSE);
	}
	else
	{
		ovcp_response_add_boolean(response, TRUE);
	}
	
	return response;
}
	

struct ovcp_response_st *node_get_traffic_hour(struct ovcp_request_st *request)
{

	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1, *arg2;
		
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE+1];

	int i = 0, ret;
		
	int nrow, ncolumn;
	char **sql_result;

	int *month, *year, *day;
	
	arg0 = ovcp_request_get_arg(request, 0);
	month = (int *)ovcp_data_get(arg0);
	
	if(!ovcp_data_is_int(arg0) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	arg1 = ovcp_request_get_arg(request, 1);
	year = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
	arg2 = ovcp_request_get_arg(request, 2);
	day = (int *)ovcp_data_get(arg2);
	
	if(!ovcp_data_is_int(arg2) || day == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
		
	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        


		
	snprintf(sql_query, BUF_SIZE, "SELECT * FROM node_hour_traffic WHERE (month=%d and day=%d and year=%d)",  *month, *day, *year);

			

	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}
	
	#ifdef DEBUG
		printf("SQL QUERY: %s\n", sql_query);
	#endif	

	i = 1;
	
	response = ovcp_response_new();

	while(i != nrow+1)
	{

		ovcp_response_struct_new(response);
		
		ovcp_response_struct_add_string(response, "month", sql_result[i*(ncolumn)+0]);
		ovcp_response_struct_add_string(response, "hour", sql_result[i*(ncolumn)+1]);
		ovcp_response_struct_add_string(response, "day", sql_result[i*(ncolumn)+2]);
		ovcp_response_struct_add_string(response, "year", sql_result[i*(ncolumn)+3]);
		ovcp_response_struct_add_string(response, "in", sql_result[i*(ncolumn)+4]);
		ovcp_response_struct_add_string(response, "out", sql_result[i*(ncolumn)+5]);
		ovcp_response_struct_add_string(response, "total", sql_result[i*(ncolumn)+6]);

		
		ovcp_response_struct_end(response);
		
		i++;	
	}
	
	sqlite3_free_table(sql_result);
				

	sqlite3_close(openvcp_db);	

	return response;
}


struct ovcp_response_st *node_get_traffic(struct ovcp_request_st *request)
{

	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1;
		
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE+1];

	int i = 0, ret;
		
	int nrow, ncolumn;
	char **sql_result;

	int *month, *year;
	
	arg0 = ovcp_request_get_arg(request, 0);
	month = (int *)ovcp_data_get(arg0);
	
	if(!ovcp_data_is_int(arg0) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	arg1 = ovcp_request_get_arg(request, 1);
	year = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		

	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        


		
	snprintf(sql_query, BUF_SIZE, "SELECT * FROM node_traffic WHERE (month=%d and year=%d)",  *month, *year);

			

	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}

	i = 1;
	
	response = ovcp_response_new();

	while(i != nrow+1)
	{

		ovcp_response_struct_new(response);
		
		ovcp_response_struct_add_string(response, "month", sql_result[i*(ncolumn)+0]);
		ovcp_response_struct_add_string(response, "day", sql_result[i*(ncolumn)+1]);
		ovcp_response_struct_add_string(response, "year", sql_result[i*(ncolumn)+2]);
		ovcp_response_struct_add_string(response, "in", sql_result[i*(ncolumn)+3]);
		ovcp_response_struct_add_string(response, "out", sql_result[i*(ncolumn)+4]);
		ovcp_response_struct_add_string(response, "total", sql_result[i*(ncolumn)+5]);

		
		ovcp_response_struct_end(response);
		
		i++;	
	}
	
	sqlite3_free_table(sql_result);
				

	sqlite3_close(openvcp_db);	

	return response;
}



struct ovcp_response_st *node_get_traffic_total(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1;
		
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE];
	int i = 0, ret;
		
	int nrow, ncolumn;
	char **sql_result;
	int *month, *year;


	double day_traffic_in, day_traffic_out, day_traffic_total;
	double month_traffic_in = 0, month_traffic_out = 0, month_traffic_total = 0;


	arg0 = ovcp_request_get_arg(request, 0);
	month = (int *)ovcp_data_get(arg0);
	
	if(!ovcp_data_is_int(arg0) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	arg1 = ovcp_request_get_arg(request, 1);
	year = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		

	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        

		
	snprintf(sql_query, BUF_SIZE, "SELECT \"in\",\"out\",\"total\" FROM node_traffic WHERE (month=%d and year=%d)",  *month, *year);

			

	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}

	i = 1;
	

	

	while(i != nrow+1)
	{
	
		day_traffic_in = strtod(sql_result[i*(ncolumn)], NULL);
		day_traffic_out = strtod(sql_result[i*(ncolumn)+1], NULL);
		day_traffic_total = strtod(sql_result[i*(ncolumn)+2], NULL);

		month_traffic_in += day_traffic_in;
		month_traffic_out += day_traffic_out;
		month_traffic_total += day_traffic_total;
		
		i++;	
	}
	

	sqlite3_free_table(sql_result);
				
	sqlite3_close(openvcp_db);	

	response = ovcp_response_new();

	ovcp_response_struct_new(response);
			
	ovcp_response_struct_add_double(response, "in", month_traffic_in);
	ovcp_response_struct_add_double(response, "out", month_traffic_out);
	ovcp_response_struct_add_double(response, "total", month_traffic_total);
	
	ovcp_response_struct_end(response);

	return response;
}


struct ovcp_response_st *node_get_interfaces(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	FILE *dev_file;
	char *word = NULL;
	int i = 0;
	
	dev_file = fopen("/proc/net/dev", "r");

	if(dev_file != NULL)
	{
		response = ovcp_response_new();
	
		ovcp_response_array_new(response);
		
		while((fscanf(dev_file, "%as", &word)) != EOF)
		{

			if(word != NULL)
			{
				i = 0;

				while(i != strlen(word))
				{
					if(word[i] == ':')
					{


						word[i] = 0;
						ovcp_response_array_add_string(response, word);

						break;
					}
	
					i++;
				}

				free(word);
			}
		}

		free(word);
		
		ovcp_response_array_end(response);
		
		fclose(dev_file);
	}
	else
	{
		response = ovcp_response_error(OVCP_ERROR_FILEMISSING, "node.get_interfaces: "OVCP_ERROR_FILEMISSING_DESC);	
	}
	
	return response;
}

struct ovcp_response_st *node_get_images(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	DIR *image_dir;
 	struct dirent *entry;
	char *path;
	struct image_st image;

	response = ovcp_response_new();
	
		
	if((image_dir = opendir(global_settings.image_dir)) != NULL)
	{

		while((entry = readdir(image_dir)) != NULL)
		{

			if(entry->d_name[0] != '.')
			{

				path = concat(global_settings.image_dir, "/", entry->d_name, NULL);

				if(dir_exist(path))
				{
					ovcp_response_struct_new(response);
					
					if(parse_image(entry->d_name, &image))
					{
						ovcp_response_struct_add_string(response, "id", entry->d_name);
						
						if(image.name != NULL)
							ovcp_response_struct_add_string(response, "name", image.name);
						if(image.initstyle != NULL)
							ovcp_response_struct_add_string(response, "initstyle", image.initstyle);
							
						free_image(&image);
					}
					else
					{
						ovcp_response_struct_add_string(response, "id", entry->d_name);
					}
					
					ovcp_response_struct_end(response);
				}

				free(path);
			}
		}

		closedir(image_dir);
	}
	

	return response;
}


struct ovcp_response_st *node_get_load(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	char path[] = "/proc/loadavg";
	char *load;
	
	load = file_read(path);
	
	if(load == NULL)
	{
		response = ovcp_response_error(OVCP_ERROR_FILEMISSING, "node.get_load: "OVCP_ERROR_FILEMISSING_DESC);
	}
	else
	{
		trim_string(load);
		response = ovcp_response_new();
		ovcp_response_add_string(response, load);
		free(load);
	}

	return response;
}


struct ovcp_response_st *node_get_uptime(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	char path[] = "/proc/uptime";
	char *uptime;
	
	uptime = file_read(path);

				
	if(uptime == NULL)
	{
		response = ovcp_response_error(OVCP_ERROR_FILEMISSING, "node.get_uptime: "OVCP_ERROR_FILEMISSING_DESC);
	}
	else
	{
		trim_string(uptime);
		response = ovcp_response_new();
		ovcp_response_add_string(response, uptime);
		free(uptime);
	}
	
	
	return response;
}


struct ovcp_response_st *node_get_space(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct statvfs vfsinfo;
	
	if(statvfs(global_settings.root_dir, &vfsinfo) == -1)
		return ovcp_response_error(OVCP_ERROR_UNABLSPACE, "node.get_space: "OVCP_ERROR_UNABLSPACE_DESC);

	
	response = ovcp_response_new();
		
	ovcp_response_struct_new(response);
			
	ovcp_response_struct_add_int(response, "space_total", vfsinfo.f_blocks/1024*vfsinfo.f_bsize);
	ovcp_response_struct_add_int(response, "space_free", vfsinfo.f_bfree/1024*vfsinfo.f_bsize);
	ovcp_response_struct_add_int(response, "inodes_total", vfsinfo.f_files);
	
	ovcp_response_struct_end(response);
		
	
	return response;
}

struct ovcp_response_st *node_get_memory(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	char path[] = "/proc/meminfo";
	char *meminfo, **lines, *word;
	unsigned int lcount;
	int i = 0;
	
	int memtotal = 0, memfree = 0, memcached = 0, membuffered = 0;
	int swaptotal = 0, swapfree = 0;
	
	meminfo = file_read(path);
	
	if(meminfo == NULL)
	{
		response = ovcp_response_error(OVCP_ERROR_FILEMISSING, "node.get_mem: "OVCP_ERROR_FILEMISSING_DESC);
	}
	else
	{
		lines = split_string(meminfo, '\n',  &lcount);
		
		while(i < lcount)
		{
			word = strtok(lines[i], " ");
			trim_string(word);
				
				
			if(strcmp(word, "MemTotal:") == 0)
			{
				word = strtok(NULL, " ");
				trim_string(word);
				memtotal = atoi(word);
			}
			else
			if(strcmp(word, "MemFree:") == 0)
			{
				word = strtok(NULL, " ");
				trim_string(word);
				memfree = atoi(word);
			}
			else
			if(strcmp(word, "Buffers:") == 0)
			{
				word = strtok(NULL, " ");
				trim_string(word);
				membuffered = atoi(word);
			}
			else
			if(strcmp(word, "Cached:") == 0)
			{
				word = strtok(NULL, " ");
				trim_string(word);
				memcached = atoi(word);
			}
			else
			if(strcmp(word, "SwapTotal:") == 0)
			{
				word = strtok(NULL, " ");
				trim_string(word);
				swaptotal = atoi(word);
			}
			else
			if(strcmp(word, "SwapFree:") == 0)
			{
				word = strtok(NULL, " ");
				trim_string(word);
				swapfree = atoi(word);
			}
			
			
		
			i++;
		}
		

		response = ovcp_response_new();
		
		ovcp_response_struct_new(response);
			
		ovcp_response_struct_add_int(response, "memtotal", memtotal);
		ovcp_response_struct_add_int(response, "memfree", memfree);
		ovcp_response_struct_add_int(response, "buffers", membuffered);
		ovcp_response_struct_add_int(response, "cached", memcached);
		ovcp_response_struct_add_int(response, "swaptotal", swaptotal);
		ovcp_response_struct_add_int(response, "swapfree", swapfree);
		
		ovcp_response_struct_end(response);
	
		free_strings(lines, lcount);
		free(meminfo);
	}
	
	

	return response;
}



