/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <syslog.h>


#include "strings.h"
#include "misc.h"
#include "main.h"
#include "request.h"
#include "error.h"
#include "logtraffic.h"
#include "fs.h"

struct ovcp_method ovcp_vserver_methods[] =
{
	{ "get_xid", get_xid },
	{ "get_state", get_state },
	{ "get_ips", get_ips },
	{ "get_flags", get_flags },
	{ "get_schedule", get_schedule },
	{ "get_hostname", get_hostname },
	{ "set_filter", set_filter },
	{ "get_filter", get_filter },
	{ "del_filter", del_filter },
	{ "get_space", get_space },
	{ "get_rlimit", get_rlimit },
	{ "get_image", get_image },
	{ "get_uptime", get_uptime },
	{ "get_mark", get_mark },
	{ "get_load", get_load },
	{ "set_rlimit", set_rlimit },
	{ "add_flag", set_flag_add },
	{ "remove_flag", set_flag_remove },
	{ "set_schedule", set_schedule },
	{ "add_iface", add_ip },
	{ "remove_iface", remove_ip },
	{ "set_space", set_space },
	{ "set_rootpw", set_rootpw },
	{ "set_hostname" , set_hostname },
	{ "set_mark" , set_mark },
	{ "remove", vserver_remove },
	{ "traffic", vserver_traffic },
	{ "traffic_total", vserver_traffic_total },
	{ "traffic_hour", vserver_traffic_hour },
	{ "start", vserver_control_start },
	{ "stop", vserver_control_stop },
	{ "get_resources", get_resources },
	{ "resources_hour", vserver_resources_hour },
	{ NULL, NULL } 
};


struct ovcp_method ovcp_node_methods[] =
{
	{ "version", node_version },
	{ "halt", node_halt },
	{ "reboot", node_reboot },	
	{ "get_interfaces", node_get_interfaces },
	{ "get_images", node_get_images },
	{ "traffic", node_get_traffic },
	{ "traffic_total", node_get_traffic_total },
	{ "traffic_hour", node_get_traffic_hour },
	{ "get_uptime", node_get_uptime },
	{ "get_load", node_get_load }, 
	{ "get_mem", node_get_memory }, 
	{ "get_space", node_get_space }, 
	{ "guest_list", vserver_list },
	{ NULL, NULL }
};




struct ovcp_method ovcp_setup_methods[] =
{	
	{ "vserver_config", vserver_setup_config },
	{ "vserver_image", vserver_setup_image },
	{ NULL, NULL }
};







struct ovcp_response_st *vserver_control_start(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0;
	char *output, *name;
	char path[BUF_SIZE];

	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);
		
		
	if(!vserver_lock(name))
		return ovcp_response_error(OVCP_ERROR_VSLOCKED, OVCP_ERROR_VSLOCKED_DESC);

	
	response = ovcp_response_new();
			
	output = exec_cmd("vserver", 30, "vserver", name, "start", NULL);

	trim_string(output);
	ovcp_response_add_string(response, output);

	vserver_unlock(name);

	free(output);
	
	
	sconcat(path, BUF_SIZE, global_settings.root_dir, "/", name, "/image-post.sh", NULL);
	

        if(file_exist(path))
	{
		output = exec_cmd("vserver", 900, "vserver", name, "exec", "/image-post.sh", NULL);
		
		if(output != NULL)
		{
			trim_string(output);	
			ovcp_response_add_string(response, output);
			free(output);
		}
		
		remove(path);
	}
		
	
	
	return response;
}

struct ovcp_response_st *vserver_control_stop(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0;
	char *output, *name;

	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);
		
		
	if(!vserver_lock(name))
		return ovcp_response_error(OVCP_ERROR_VSLOCKED, OVCP_ERROR_VSLOCKED_DESC);

	
	response = ovcp_response_new();
			
	output = exec_cmd("vserver", 30, "vserver", name, "stop", NULL);
	
	trim_string(output);
	ovcp_response_add_string(response, output);

	vserver_unlock(name);

	free(output);
	
	return response;
}





struct ovcp_response_st *vserver_list(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	DIR *dir_ptr;
	struct dirent *dir_entry;
	char path[BUF_SIZE];

	response = ovcp_response_new();
	
	
	ovcp_response_array_new(response);

	if((dir_ptr = opendir(global_settings.config_dir)) != NULL)
	{
		
		while((dir_entry = readdir(dir_ptr)) != NULL)
		{
			

			if(dir_entry->d_name[0] != '.')
			{
				sconcat(path, BUF_SIZE, global_settings.config_dir, "/", dir_entry->d_name, NULL);

				if(dir_exist(path))
					ovcp_response_array_add_string(response, dir_entry->d_name);
			}
	
		}


		closedir(dir_ptr);
	}

	ovcp_response_array_end(response);
		
	return response;
}

struct ovcp_response_st *vserver_resources_hour(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1, *arg2, *arg3;
	int i, ret, *month, *year, *day, nrow, ncolumn;
	char *name;
	
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE];
	char **sql_result;
		
	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);	

	arg1 = ovcp_request_get_arg(request, 1);
	month = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);	

	arg2 = ovcp_request_get_arg(request, 2);
	year = (int *)ovcp_data_get(arg2);
	
	if(!ovcp_data_is_int(arg2) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
	arg3 = ovcp_request_get_arg(request, 3);
	day = (int *)ovcp_data_get(arg3);
	
	if(!ovcp_data_is_int(arg3) || day == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
		
		
	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        
	
	
	snprintf(sql_query, BUF_SIZE, 
		"SELECT * FROM vserver_hour_resources WHERE (name='%s' and month=%d and day=%d and year=%d)", name, *month, *day, *year);

	#ifdef DEBUG
		printf("SQL Traffic Query: [%s]\n", sql_query);
	#endif

			

	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}

	i = 1;
	response = ovcp_response_new();

	while(i != nrow+1)
	{
		ovcp_response_struct_new(response);
			
	
		ovcp_response_struct_add_string(response, "month", sql_result[i*(ncolumn)+0]);
		ovcp_response_struct_add_string(response, "hour", sql_result[i*(ncolumn)+1]);
		ovcp_response_struct_add_string(response, "day", sql_result[i*(ncolumn)+2]);
		ovcp_response_struct_add_string(response, "year", sql_result[i*(ncolumn)+3]);
		ovcp_response_struct_add_string(response, "proc", sql_result[i*(ncolumn)+5]);
		ovcp_response_struct_add_string(response, "vm", sql_result[i*(ncolumn)+6]);
		ovcp_response_struct_add_string(response, "vml", sql_result[i*(ncolumn)+7]);
		ovcp_response_struct_add_string(response, "rss", sql_result[i*(ncolumn)+8]);
		ovcp_response_struct_add_string(response, "anon", sql_result[i*(ncolumn)+9]);
		ovcp_response_struct_add_string(response, "files", sql_result[i*(ncolumn)+10]);
		ovcp_response_struct_add_string(response, "ofd", sql_result[i*(ncolumn)+11]);
		ovcp_response_struct_add_string(response, "locks", sql_result[i*(ncolumn)+12]);
		ovcp_response_struct_add_string(response, "sock", sql_result[i*(ncolumn)+13]);
		ovcp_response_struct_add_string(response, "msgq", sql_result[i*(ncolumn)+14]);
		ovcp_response_struct_add_string(response, "shm", sql_result[i*(ncolumn)+15]);
		
		
		ovcp_response_struct_end(response);

		i++;	
	}


	sqlite3_free_table(sql_result);
				

	sqlite3_close(openvcp_db);	


	return response;

}


struct ovcp_response_st *vserver_traffic_hour(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1, *arg2, *arg3;
	
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE+1];

	char *name;
	int ret;
	int i = 0;
		
	int nrow, ncolumn;
	char **sql_result;
	int *month, *year, *day;
	
	char *ddn_ip;
	ovcp_ip ipdec;
			
	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);	
	
	arg1 = ovcp_request_get_arg(request, 1);
	month = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	arg2 = ovcp_request_get_arg(request, 2);
	year = (int *)ovcp_data_get(arg2);
	
	if(!ovcp_data_is_int(arg2) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
	arg3 = ovcp_request_get_arg(request, 3);
	day = (int *)ovcp_data_get(arg3);
	
	if(!ovcp_data_is_int(arg3) || day == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
		
	
	
	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        


		
	snprintf(sql_query, BUF_SIZE, 
		"SELECT * FROM vserver_hour_traffic WHERE (month=%d and day=%d and year=%d and name='%s')", *month, *day, *year, name);

	#ifdef DEBUG
		printf("SQL Traffic Query: [%s]\n", sql_query);
	#endif
	
			

	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}

	i = 1;
	response = ovcp_response_new();

	while(i != nrow+1)
	{
		ovcp_response_struct_new(response);
		
			
		#ifdef IPV6_SUPPORT
		if(strstr(sql_result[i*(ncolumn)+4], ":") != NULL)
		{
			ddn_ip = strdup(sql_result[i*(ncolumn)+4]);
		}
		else
		{
			sscanf(sql_result[i*(ncolumn)+4], "%u", (unsigned int *)&ipdec);
			ddn_ip = ip2str(*((uint32_t *)&ipdec));
		}
		#else
		sscanf(sql_result[i*(ncolumn)+4], "%u", &ipdec);
		ddn_ip = ip2str(ipdec);
		#endif
		
		ovcp_response_struct_add_string(response, "month", sql_result[i*(ncolumn)+0]);
		ovcp_response_struct_add_string(response, "hour", sql_result[i*(ncolumn)+1]);
		ovcp_response_struct_add_string(response, "day", sql_result[i*(ncolumn)+2]);
		ovcp_response_struct_add_string(response, "year", sql_result[i*(ncolumn)+3]);
		ovcp_response_struct_add_string(response, "ip", ddn_ip);
		ovcp_response_struct_add_string(response, "in", sql_result[i*(ncolumn)+5]);
		ovcp_response_struct_add_string(response, "out", sql_result[i*(ncolumn)+6]);
		ovcp_response_struct_add_string(response, "total", sql_result[i*(ncolumn)+7]);
		
		free(ddn_ip);
		
		ovcp_response_struct_end(response);

		i++;	
	}




	sqlite3_free_table(sql_result);
				

	sqlite3_close(openvcp_db);	


	return response;
}



struct ovcp_response_st *vserver_traffic(struct ovcp_request_st *request)
{

	char *ddn_ip;
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1, *arg2;
	
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE+1];

	char *name;
	int ret;
	int i = 0;
		
	int nrow, ncolumn;
	char **sql_result;
	int *month, *year;
	
	ovcp_ip ipdec;
			
	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);	
	
	arg1 = ovcp_request_get_arg(request, 1);
	month = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	arg2 = ovcp_request_get_arg(request, 2);
	year = (int *)ovcp_data_get(arg2);
	
	if(!ovcp_data_is_int(arg2) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
	
	
	
	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        

		
	snprintf(sql_query, BUF_SIZE, 
		"SELECT * FROM vserver_traffic WHERE (month=%d and year=%d and name='%s')", *month, *year, name);

	#ifdef DEBUG
		printf("SQL Traffic Query: [%s]\n", sql_query);
	#endif

			

	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}

	i = 1;
	response = ovcp_response_new();

	while(i != nrow+1)
	{
		ovcp_response_struct_new(response);
		

		#ifdef IPV6_SUPPORT
		if(strstr(sql_result[i*(ncolumn)+3], ":") != NULL)
		{
			ddn_ip = strdup(sql_result[i*(ncolumn)+3]);
		}
		else
		{
			sscanf(sql_result[i*(ncolumn)+3], "%u", (unsigned int *)&ipdec);
			ddn_ip = ip2str(*((uint32_t *)&ipdec));
		}
		#else
		sscanf(sql_result[i*(ncolumn)+3], "%u", &ipdec);
		ddn_ip = ip2str(ipdec);
		#endif
		
		ovcp_response_struct_add_string(response, "month", sql_result[i*(ncolumn)+0]);
		ovcp_response_struct_add_string(response, "day", sql_result[i*(ncolumn)+1]);
		ovcp_response_struct_add_string(response, "year", sql_result[i*(ncolumn)+2]);
		ovcp_response_struct_add_string(response, "ip", ddn_ip);
		ovcp_response_struct_add_string(response, "in", sql_result[i*(ncolumn)+4]);
		ovcp_response_struct_add_string(response, "out", sql_result[i*(ncolumn)+5]);
		ovcp_response_struct_add_string(response, "total", sql_result[i*(ncolumn)+6]);

		
		free(ddn_ip);
			
		ovcp_response_struct_end(response);

		i++;	
	}




	sqlite3_free_table(sql_result);
				

	sqlite3_close(openvcp_db);	


	return response;
}




struct ovcp_response_st *vserver_traffic_total(struct ovcp_request_st *request)
{

	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1, *arg2;
	
	sqlite3 *openvcp_db;
	char *sql_error; 
	char sql_query[BUF_SIZE+1];

	char *name;
	int ret;
	int i = 0;
		
	int nrow, ncolumn;
	char **sql_result;
	
	double day_traffic_in, day_traffic_out, day_traffic_total;
	double month_traffic_in = 0, month_traffic_out = 0, month_traffic_total = 0;
	int *month, *year;
	
	
	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);	
	
	arg1 = ovcp_request_get_arg(request, 1);
	month = (int *)ovcp_data_get(arg1);
	
	if(!ovcp_data_is_int(arg1) || month == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	arg2 = ovcp_request_get_arg(request, 2);
	year = (int *)ovcp_data_get(arg2);
	
	if(!ovcp_data_is_int(arg2) || year == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
		
	
	
		
	ret = sqlite3_open(OPENVCP_DATABASE, &openvcp_db);

	if(ret != 0)
	{
		syslog(LOG_ERR, "Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}        



		
	snprintf(sql_query, BUF_SIZE, 
		"SELECT \"in\",\"out\",\"total\" FROM vserver_traffic WHERE (month=%d and year=%d and name='%s')", *month, *year, name);
	#ifdef DEBUG
		printf("SQL Traffic Query: [%s]\n", sql_query);
	#endif


	
			

	
	ret = sqlite3_get_table(openvcp_db, sql_query, &sql_result, &nrow, &ncolumn, &sql_error);

	if(ret != SQLITE_OK)
	{
		syslog(LOG_ERR, "SQL error (%s)", sql_error);
		sqlite3_free(sql_error);
		sqlite3_close(openvcp_db);
		exit(-1);
	}

	i = 1;
	
	while(i != nrow+1)
	{
	
		
		day_traffic_in = strtod(sql_result[i*(ncolumn)], NULL);
		day_traffic_out = strtod(sql_result[i*(ncolumn)+1], NULL);
		day_traffic_total = strtod(sql_result[i*(ncolumn)+2], NULL);


		month_traffic_in += day_traffic_in;
		month_traffic_out += day_traffic_out;
		month_traffic_total += day_traffic_total;
	
		i++;	
	}
	
	sqlite3_free_table(sql_result);
				
	sqlite3_close(openvcp_db);	

	response = ovcp_response_new();
	
	ovcp_response_struct_new(response);
			
	ovcp_response_struct_add_double(response, "in", month_traffic_in);
	ovcp_response_struct_add_double(response, "out", month_traffic_out);
	ovcp_response_struct_add_double(response, "total", month_traffic_total);
	
	ovcp_response_struct_end(response);

	
	return response;
}

struct ovcp_response_st *vserver_setup_config(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0;
	char *name;

	
	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	if(setup_config(name) != 1)
		return ovcp_response_error(OVCP_ERROR_UABLCRTCONF, OVCP_ERROR_UABLCRTCONF_DESC);

	response = ovcp_response_new();
	
	ovcp_response_add_boolean(response, TRUE);

	return response;
}

struct ovcp_response_st *vserver_setup_image(struct ovcp_request_st *request)
{

	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0, *arg1;
	char *name, *image, *output;
	char path1[BUF_SIZE], path2[BUF_SIZE];

	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);
		
	if(!vserver_lock(name))
		return ovcp_response_error(OVCP_ERROR_VSLOCKED, OVCP_ERROR_VSLOCKED_DESC);


	arg1 = ovcp_request_get_arg(request, 1);
	image = (char *)ovcp_data_get(arg1);
		
	if(!ovcp_data_is_string(arg1) || image == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);

	
	if(setup_image(name, image) != 1)
		return  ovcp_response_error(OVCP_ERROR_UABLCPYIMG, OVCP_ERROR_UABLCPYIMG_DESC);


	sconcat(path1, BUF_SIZE, global_settings.image_dir, "/", image, "-post.sh", NULL);

	if(file_exist(path1))
	{
		sconcat(path2, BUF_SIZE, global_settings.root_dir, "/", name, "/image-post.sh", NULL);
		copy(path1, path2);
	}



	response = ovcp_response_new();	

	sconcat(path1, BUF_SIZE, global_settings.image_dir, "/", image, "-pre.sh", NULL);
	

        if(file_exist(path1))
	{
		output = exec_cmd("sh", 900, "sh", path1, name, NULL);
		
		if(output != NULL)
		{
			ovcp_response_add_string(response, output);
			free(output);
		}
		else
		{
			ovcp_response_add_boolean(response, TRUE);
		}
	}
	else
	{
		ovcp_response_add_boolean(response, TRUE);	
	}	
	
	

	
	vserver_unlock(name);

	

	return response;
}

struct ovcp_response_st *vserver_remove(struct ovcp_request_st *request)
{
	struct ovcp_response_st *response;
	struct ovcp_data_st *arg0;
	char *name,  *path, *output;

	arg0 = ovcp_request_get_arg(request, 0);
	name = (char *)ovcp_data_get(arg0);
		
	if(!ovcp_data_is_string(arg0) || name == NULL)
		return ovcp_response_error(OVCP_ERROR_ARGMISS, OVCP_ERROR_ARGMISS_DESC);
	
	if(!vserver_exist(name))
		return ovcp_response_error(OVCP_ERROR_VSNOTEXIST, OVCP_ERROR_VSNOTEXIST_DESC);
	
	
	path = concat(global_settings.config_dir, "/", name, NULL);

	if(dir_exist(path))
	{
		output = exec_cmd("rm", 10, "rm", "-rf", path, NULL);
		free(output);
	}
		
	free(path);
	path = concat(global_settings.root_dir, "/", name, NULL);
	
	if(dir_exist(path))
	{
		output = exec_cmd("rm", 180, "rm", "-rf", path, NULL);
		free(output);
	}

	free(path);	
		
	response = ovcp_response_new();	
	ovcp_response_add_boolean(response, TRUE);


	return response;
}

